@@ -12,16 +12,13 @@
 #
 ##############################################################################
 
-import platform
 import re
-import sys
 import unittest
 
-import persistent
-from persistent._compat import copy_reg
-
-
-_is_pypy3 = platform.python_implementation() == 'PyPy' and sys.version_info[0] > 2
+import copyreg
+from persistent._compat import PYPY
+from persistent.tests.utils import skipIfNoCExtension
+from collections import UserDict as IterableUserDict
 
 # pylint:disable=R0904,W0212,E1101
 # pylint:disable=attribute-defined-outside-init,too-many-lines
@@ -29,18 +26,8 @@ _is_pypy3 = platform.python_implementation() == 'PyPy' and sys.version_info[0] >
 # Hundreds of unused jar and OID vars make this useless
 # pylint:disable=unused-variable
 
-def skipIfNoCExtension(o):
-    return unittest.skipIf(
-        persistent._cPersistence is None,
-        "The C extension is not available")(o)
 
-
-class _Persistent_Base(object):
-
-    # py2/3 compat
-    assertRaisesRegex = getattr(unittest.TestCase,
-                                'assertRaisesRegex',
-                                unittest.TestCase.assertRaisesRegexp)
+class _Persistent_Base:
 
     def _getTargetClass(self):
         # concrete testcase classes must override
@@ -56,12 +43,12 @@ class _Persistent_Base(object):
     def _makeOne(self, *args, **kw):
         return self._getTargetClass()(*args, **kw)
 
-    def _makeJar(self):
+    def _makeJar(self, real_cache=False):
         from zope.interface import implementer
         from persistent.interfaces import IPersistentDataManager
 
         @implementer(IPersistentDataManager)
-        class _Jar(object):
+        class _Jar:
             _cache = None
             # Set this to a value to have our `setstate`
             # pass it through to the object's __setstate__
@@ -83,7 +70,7 @@ class _Persistent_Base(object):
                 self._registered.append(obj._p_oid)
 
         jar = _Jar()
-        jar._cache = self._makeCache(jar)
+        jar._cache = self._makeRealCache(jar) if real_cache else self._makeCache(jar)
         return jar
 
     def _makeBrokenJar(self):
@@ -91,7 +78,7 @@ class _Persistent_Base(object):
         from persistent.interfaces import IPersistentDataManager
 
         @implementer(IPersistentDataManager)
-        class _BrokenJar(object):
+        class _BrokenJar:
             def __init__(self):
                 self.called = 0
             def register(self, ob):
@@ -104,14 +91,16 @@ class _Persistent_Base(object):
         jar._cache = self._makeCache(jar)
         return jar
 
-    def _makeOneWithJar(self, klass=None, broken_jar=False):
+    def _makeOneWithJar(self, klass=None, broken_jar=False, real_cache=False):
         OID = b'\x01' * 8
         if klass is not None:
             inst = klass()
         else:
             inst = self._makeOne()
-        jar = self._makeJar() if not broken_jar else self._makeBrokenJar()
+        jar = self._makeJar(real_cache=real_cache) if not broken_jar else self._makeBrokenJar()
         jar._cache.new_ghost(OID, inst) # assigns _p_jar, _p_oid
+        # Be sure it really returned a ghost.
+        assert inst._p_status == 'ghost'
         return inst, jar, OID
 
     def test_class_conforms_to_IPersistent(self):
@@ -124,6 +113,14 @@ class _Persistent_Base(object):
         from persistent.interfaces import IPersistent
         verifyObject(IPersistent, self._makeOne())
 
+    def test_instance_cannot_be_weakly_referenced(self):
+        if PYPY: # pragma: no cover
+            self.skipTest('On PyPy, everything can be weakly referenced')
+        import weakref
+        inst = self._makeOne()
+        with self.assertRaises(TypeError):
+            weakref.ref(inst)
+
     def test_ctor(self):
         from persistent.persistence import _INITIAL_SERIAL
         inst = self._makeOne()
@@ -257,7 +254,7 @@ class _Persistent_Base(object):
     def test_delete_p_oid_of_subclass_calling_p_delattr(self):
         class P(self._getTargetClass()):
             def __delattr__(self, name):
-                super(P, self)._p_delattr(name)
+                super()._p_delattr(name)
                 raise AssertionError("Should not get here")
 
         inst, _jar, _oid = self._makeOneWithJar(klass=P)
@@ -731,11 +728,11 @@ class _Persistent_Base(object):
         # This comes from the C implementation and is maintained
         # for backwards compatibility. (For example, Persistent and
         # ExtensionClass.Base/Acquisition take special care to mix together.)
-        class Base(object):
+        class Base:
             def __getattribute__(self, name):
                 if name == 'magic':
                     return 42
-                return super(Base, self).__getattribute__(name) # pragma: no cover
+                return super().__getattribute__(name) # pragma: no cover
 
         self.assertEqual(getattr(Base(), 'magic'), 42)
 
@@ -801,6 +798,51 @@ class _Persistent_Base(object):
         self.assertEqual(getattr(inst, 'normal', None), 'after')
         self.assertEqual(inst._p_status, 'changed')
 
+    def test__setattr__class__from_saved(self):
+        # Setting __class__ activates the object and uses
+        # the old class's methods to do so. See
+        # https://github.com/zopefoundation/persistent/issues/155
+        P = self._getTargetClass()
+
+        setstate = []
+
+        class OriginalClass(P):
+            def __setstate__(self, state):
+                setstate.append(type(self))
+                P.__setstate__(self, state)
+
+        class NewClass(P):
+            "Does nothing"
+
+        inst, jar, OID = self._makeOneWithJar(OriginalClass)
+        # Make the fake jar call __setstate__ when the object is activated
+        jar.setstate_calls_object = {}
+        inst._p_changed = False
+        self._clearMRU(jar)
+
+        # Assigning to __class__...
+        inst.__class__ = NewClass
+        # ...uses the original __setstate__ method...
+        self.assertEqual(setstate, [OriginalClass])
+        # ...and activates the object
+        self.assertTrue(inst._p_changed)
+
+    def test__setattr__dict__from_saved(self):
+        # Setting __dict__ activates the object and uses
+        # the old class's methods to do so. See
+        # https://github.com/zopefoundation/persistent/issues/155
+        class Derived(self._getTargetClass()):
+            "Nothing special"
+
+        inst, jar, OID = self._makeOneWithJar(Derived)
+        inst._p_changed = False
+        self._clearMRU(jar)
+
+        # Assigning to __dict__...
+        inst.__dict__ = {}
+        # ...activates the object
+        self.assertTrue(inst._p_changed)
+
     def test___setattr__normal_name_from_changed(self):
         class Derived(self._getTargetClass()):
             normal = 'before'
@@ -835,10 +877,10 @@ class _Persistent_Base(object):
         delattr(inst, 'normal')
         self.assertEqual(getattr(inst, 'normal', None), 'before')
 
-    def test___delattr__normal_name_from_ghost(self):
+    def test___delattr__normal_name_from_ghost(self, real_cache=False):
         class Derived(self._getTargetClass()):
             normal = 'before'
-        inst, jar, OID = self._makeOneWithJar(Derived)
+        inst, jar, OID = self._makeOneWithJar(Derived, real_cache=real_cache)
         inst._p_deactivate()
         self._clearMRU(jar)
         jar._registered = []
@@ -850,12 +892,15 @@ class _Persistent_Base(object):
         self.assertEqual(jar._registered, [OID])
         self.assertEqual(getattr(inst, 'normal', None), 'before')
 
-    def test___delattr__normal_name_from_saved(self):
+    def test___delattr__normal_name_from_ghost_real_cache(self):
+        self.test___delattr__normal_name_from_ghost(real_cache=True)
+
+    def test___delattr__normal_name_from_saved(self, real_cache=False):
         class Derived(self._getTargetClass()):
             normal = 'before'
             def __init__(self):
                 self.__dict__['normal'] = 'after'
-        inst, jar, OID = self._makeOneWithJar(Derived)
+        inst, jar, OID = self._makeOneWithJar(Derived, real_cache=real_cache)
         inst._p_changed = False
         self._clearMRU(jar)
         jar._registered = []
@@ -864,12 +909,15 @@ class _Persistent_Base(object):
         self.assertEqual(jar._registered, [OID])
         self.assertEqual(getattr(inst, 'normal', None), 'before')
 
-    def test___delattr__normal_name_from_changed(self):
+    def test___delattr__normal_name_from_saved_real_cache(self):
+        self.test___delattr__normal_name_from_saved(real_cache=True)
+
+    def test___delattr__normal_name_from_changed(self, real_cache=False):
         class Derived(self._getTargetClass()):
             normal = 'before'
             def __init__(self):
                 self.__dict__['normal'] = 'after'
-        inst, jar, OID = self._makeOneWithJar(Derived)
+        inst, jar, OID = self._makeOneWithJar(Derived, real_cache=real_cache)
         inst._p_changed = True
         self._clearMRU(jar)
         jar._registered = []
@@ -878,6 +926,9 @@ class _Persistent_Base(object):
         self.assertEqual(jar._registered, [])
         self.assertEqual(getattr(inst, 'normal', None), 'before')
 
+    def test___delattr__normal_name_from_changed_real_cache(self):
+        self.test___delattr__normal_name_from_changed(real_cache=True)
+
     def test___getstate__(self):
         inst = self._makeOne()
         self.assertEqual(inst.__getstate__(), None)
@@ -984,7 +1035,7 @@ class _Persistent_Base(object):
         self.assertEqual(inst.baz, 'bam')
         self.assertEqual(inst.qux, 'spam')
 
-    if not _is_pypy3:
+    if not PYPY:
         def test___setstate___interns_dict_keys(self):
             class Derived(self._getTargetClass()):
                 pass
@@ -999,7 +1050,6 @@ class _Persistent_Base(object):
             key2 = list(inst2.__dict__.keys())[0]
             self.assertTrue(key1 is key2)
 
-            from persistent._compat import IterableUserDict
             inst1 = Derived()
             inst2 = Derived()
             key1 = 'key'
@@ -1034,7 +1084,6 @@ class _Persistent_Base(object):
             pass
         inst1 = Derived()
 
-        from persistent._compat import IterableUserDict
         state = IterableUserDict({'foobar': [1, 2]})
 
         inst1.__setstate__(state)
@@ -1043,7 +1092,7 @@ class _Persistent_Base(object):
     def test___reduce__(self):
         inst = self._makeOne()
         first, second, third = inst.__reduce__()
-        self.assertTrue(first is copy_reg.__newobj__)
+        self.assertTrue(first is copyreg.__newobj__)
         self.assertEqual(second, (self._getTargetClass(),))
         self.assertEqual(third, None)
 
@@ -1053,7 +1102,7 @@ class _Persistent_Base(object):
                 return ('a', 'b')
         inst = Derived()
         first, second, third = inst.__reduce__()
-        self.assertTrue(first is copy_reg.__newobj__)
+        self.assertTrue(first is copyreg.__newobj__)
         self.assertEqual(second, (Derived, 'a', 'b'))
         self.assertEqual(third, {})
 
@@ -1063,7 +1112,7 @@ class _Persistent_Base(object):
                 return {}
         inst = Derived()
         first, second, third = inst.__reduce__()
-        self.assertTrue(first is copy_reg.__newobj__)
+        self.assertTrue(first is copyreg.__newobj__)
         self.assertEqual(second, (Derived,))
         self.assertEqual(third, {})
 
@@ -1075,14 +1124,25 @@ class _Persistent_Base(object):
                 return {'foo': 'bar'}
         inst = Derived()
         first, second, third = inst.__reduce__()
-        self.assertTrue(first is copy_reg.__newobj__)
+        self.assertTrue(first is copyreg.__newobj__)
         self.assertEqual(second, (Derived, 'a', 'b'))
         self.assertEqual(third, {'foo': 'bar'})
 
+    def _get_cucumber(self, name):
+        # Checks that it's actually a subclass of what we're testing;
+        # if it isn't, the test is skipped. The cucumbers module can
+        # only subclass either the C or Python implementation, not
+        # both
+        from persistent.tests import cucumbers
+        cls = getattr(cucumbers, name)
+        if not issubclass(cls, self._getTargetClass()):
+            self.skipTest("Cucumber is not correct implementation")
+        return cls
+
     def test_pickle_roundtrip_simple(self):
         import pickle
         # XXX s.b. 'examples'
-        from persistent.tests.cucumbers import Simple
+        Simple = self._get_cucumber('Simple')
         inst = Simple('testing')
         copy = pickle.loads(pickle.dumps(inst))
         self.assertEqual(copy, inst)
@@ -1093,7 +1153,7 @@ class _Persistent_Base(object):
     def test_pickle_roundtrip_w_getnewargs_and_getstate(self):
         import pickle
         # XXX s.b. 'examples'
-        from persistent.tests.cucumbers import Custom
+        Custom = self._get_cucumber('Custom')
         inst = Custom('x', 'y')
         copy = pickle.loads(pickle.dumps(inst))
         self.assertEqual(copy, inst)
@@ -1104,7 +1164,7 @@ class _Persistent_Base(object):
     def test_pickle_roundtrip_w_slots_missing_slot(self):
         import pickle
         # XXX s.b. 'examples'
-        from persistent.tests.cucumbers import SubSlotted
+        SubSlotted = self._get_cucumber('SubSlotted')
         inst = SubSlotted('x', 'y', 'z')
         copy = pickle.loads(pickle.dumps(inst))
         self.assertEqual(copy, inst)
@@ -1115,7 +1175,7 @@ class _Persistent_Base(object):
     def test_pickle_roundtrip_w_slots_filled_slot(self):
         import pickle
         # XXX s.b. 'examples'
-        from persistent.tests.cucumbers import SubSlotted
+        SubSlotted = self._get_cucumber('SubSlotted')
         inst = SubSlotted('x', 'y', 'z')
         inst.s4 = 'a'
         copy = pickle.loads(pickle.dumps(inst))
@@ -1127,7 +1187,7 @@ class _Persistent_Base(object):
     def test_pickle_roundtrip_w_slots_and_empty_dict(self):
         import pickle
         # XXX s.b. 'examples'
-        from persistent.tests.cucumbers import SubSubSlotted
+        SubSubSlotted = self._get_cucumber('SubSubSlotted')
         inst = SubSubSlotted('x', 'y', 'z')
         copy = pickle.loads(pickle.dumps(inst))
         self.assertEqual(copy, inst)
@@ -1138,7 +1198,7 @@ class _Persistent_Base(object):
     def test_pickle_roundtrip_w_slots_and_filled_dict(self):
         import pickle
         # XXX s.b. 'examples'
-        from persistent.tests.cucumbers import SubSubSlotted
+        SubSubSlotted = self._get_cucumber('SubSubSlotted')
         inst = SubSubSlotted('x', 'y', 'z', foo='bar', baz='bam')
         inst.s4 = 'a'
         copy = pickle.loads(pickle.dumps(inst))
@@ -1362,22 +1422,39 @@ class _Persistent_Base(object):
 
     def test__p_invalidate_from_changed_w_slots(self):
         class Derived(self._getTargetClass()):
-            __slots__ = ('myattr1', 'myattr2', 'unset')
+            __slots__ = {
+                'myattr1': 'value1',
+                'myattr2': 'value2',
+                'unset': None
+            }
+
             def __init__(self):
                 self.myattr1 = 'value1'
                 self.myattr2 = 'value2'
+
         inst, jar, OID = self._makeOneWithJar(Derived)
         inst._p_activate()
         inst._p_changed = True
         jar._loaded = []
         jar._registered = []
-        self.assertEqual(Derived.myattr1.__get__(inst), 'value1')
-        self.assertEqual(Derived.myattr2.__get__(inst), 'value2')
+        for slot, expected_value in Derived.__slots__.items():
+            slot_descriptor = getattr(Derived, slot)
+            if expected_value:
+                self.assertEqual(slot_descriptor.__get__(inst), expected_value)
+            else:
+                with self.assertRaises(AttributeError):
+                    slot_descriptor.__get__(inst)
+
         inst._p_invalidate()
         self.assertEqual(inst._p_status, 'ghost')
         self.assertEqual(list(jar._loaded), [])
-        self.assertRaises(AttributeError, lambda: Derived.myattr1.__get__(inst))
-        self.assertRaises(AttributeError, lambda: Derived.myattr2.__get__(inst))
+
+        for slot in Derived.__slots__:
+            __traceback_info__ = slot, inst
+            slot_descriptor = getattr(Derived, slot)
+            with self.assertRaises(AttributeError):
+                slot_descriptor.__get__(inst)
+
         self.assertEqual(list(jar._loaded), [])
         self.assertEqual(list(jar._registered), [])
 
@@ -1420,10 +1497,10 @@ class _Persistent_Base(object):
             __slots__ = ('slot1',)
 
         # Pre-cache in __slotnames__; cpersistent goes directly for this
-        # and avoids a call to copy_reg. (If it calls the python code in
-        # copy_reg, the pending exception will be immediately propagated by
-        # copy_reg, not by us.)
-        copy_reg._slotnames(Derived)
+        # and avoids a call to copyreg. (If it calls the python code in
+        # copyreg, the pending exception will be immediately propagated by
+        # copyreg, not by us.)
+        copyreg._slotnames(Derived)
 
         inst, jar, OID = self._makeOneWithJar(Derived, broken_jar=True)
         inst._p_invalidate()
@@ -1595,7 +1672,7 @@ class _Persistent_Base(object):
             pass
         class C(A, B):
             pass
-        class D(object):
+        class D:
             pass
         class E(D, B):
             pass
@@ -1605,7 +1682,7 @@ class _Persistent_Base(object):
     def test_w_alternate_metaclass(self):
         class alternateMeta(type):
             pass
-        class alternate(object):
+        class alternate:
             __metaclass__ = alternateMeta
         class mixedMeta(alternateMeta, type):
             pass
@@ -1629,7 +1706,7 @@ class _Persistent_Base(object):
         class Broken(self._getTargetClass()):
             def __setattr__(self, name, value):
                 if name.startswith('_p_'):
-                    super(Broken, self).__setattr__(name, value)
+                    super().__setattr__(name, value)
                 else:
                     raise AssertionError("Can't change broken objects")
 
@@ -1719,7 +1796,7 @@ class _Persistent_Base(object):
     def test_repr_no_oid_in_jar(self):
         p = self._makeOne()
 
-        class Jar(object):
+        class Jar:
             def __repr__(self):
                 return '<SomeJar>'
 
@@ -1757,7 +1834,7 @@ class _Persistent_Base(object):
     def test_repr_no_oid_repr_jar_raises_exception(self):
         p = self._makeOne()
 
-        class Jar(object):
+        class Jar:
             def __repr__(self):
                 raise Exception('jar repr failed')
 
@@ -1801,7 +1878,7 @@ class _Persistent_Base(object):
                 raise Exception("oid repr failed")
         p._p_oid = BadOID(b'1234567')
 
-        class Jar(object):
+        class Jar:
             def __repr__(self):
                 raise Exception('jar repr failed')
 
@@ -1817,7 +1894,7 @@ class _Persistent_Base(object):
     def test_repr_no_oid_repr_jar_raises_baseexception(self):
         p = self._makeOne()
 
-        class Jar(object):
+        class Jar:
             def __repr__(self):
                 raise BaseException('jar repr failed')
 
@@ -1845,7 +1922,7 @@ class _Persistent_Base(object):
         p = self._makeOne()
         p._p_oid = self._PACKED_OID
 
-        class Jar(object):
+        class Jar:
             def __repr__(self):
                 return '<SomeJar>'
 
@@ -1881,7 +1958,7 @@ class _Persistent_Base(object):
             "<persistent.tests.test_persistence.P object at 0xdeadbeef oid " + self._HEX_OID
             + " _p_repr Exception('_p_repr failed')>")
 
-        class Jar(object):
+        class Jar:
             def __repr__(self):
                 return '<SomeJar>'
 
@@ -1912,12 +1989,13 @@ class _Persistent_Base(object):
 class PyPersistentTests(unittest.TestCase, _Persistent_Base):
 
     def _getTargetClass(self):
-        from persistent.persistence import Persistent
-        return Persistent
+        from persistent.persistence import PersistentPy
+        assert PersistentPy.__module__ == 'persistent.persistence', PersistentPy.__module__
+        return PersistentPy
 
     def _makeCache(self, jar):
 
-        class _Cache(object):
+        class _Cache:
             def __init__(self, jar):
                 self._jar = jar
                 self._mru = []
@@ -1927,6 +2005,14 @@ class PyPersistentTests(unittest.TestCase, _Persistent_Base):
             def new_ghost(self, oid, obj):
                 obj._p_jar = self._jar
                 obj._p_oid = oid
+                # The C implementation always returns actual ghosts,
+                # make sure we do too. However, we can't call
+                # _p_deactivate(), because that clears the dictionary.
+                # The C pickle cache makes the object a ghost just by
+                # setting its status to 'ghost', without going though
+                # _p_deactivate(). Thus, we do the same by setting the
+                # flags.
+                object.__setattr__(obj, '_Persistent__flags', None)
                 self._data[oid] = obj
             def get(self, oid):
                 return self._data.get(oid)
@@ -1937,15 +2023,23 @@ class PyPersistentTests(unittest.TestCase, _Persistent_Base):
 
         return _Cache(jar)
 
+    def _getRealCacheClass(self):
+        from persistent.picklecache import PickleCachePy as PickleCache
+        return PickleCache
+
     def _makeRealCache(self, jar):
-        from persistent.picklecache import PickleCache
+        PickleCache = self._getRealCacheClass()
         return PickleCache(jar, 10)
 
     def _checkMRU(self, jar, value):
-        self.assertEqual(list(jar._cache._mru), value)
+        if not isinstance(jar._cache, self._getRealCacheClass()):
+            # We can't do this for the real cache.
+            self.assertEqual(list(jar._cache._mru), value)
 
     def _clearMRU(self, jar):
-        jar._cache._mru[:] = []
+        if not isinstance(jar._cache, self._getRealCacheClass()):
+            # We can't do this for the real cache.
+            jar._cache._mru[:] = []
 
     def test_accessed_with_jar_and_oid_but_not_in_cache(self):
         # This scenario arises in ZODB: ZODB.serialize.ObjectWriter
@@ -1971,7 +2065,7 @@ class PyPersistentTests(unittest.TestCase, _Persistent_Base):
     def test_accessed_invalidated_with_jar_and_oid_but_no_cache(self):
         # This scenario arises in ZODB tests where the jar is faked
         KEY = b'123'
-        class Jar(object):
+        class Jar:
             accessed = False
             def __getattr__(self, name):
                 if name == '_cache':
@@ -2024,8 +2118,8 @@ class PyPersistentTests(unittest.TestCase, _Persistent_Base):
 class CPersistentTests(unittest.TestCase, _Persistent_Base):
 
     def _getTargetClass(self):
-        from persistent.cPersistence import Persistent
-        return Persistent
+        from persistent._compat import _c_optimizations_available as get_c
+        return get_c()['persistent.persistence'].Persistent
 
     def _checkMRU(self, jar, value):
         pass # Figure this out later
@@ -2034,7 +2128,8 @@ class CPersistentTests(unittest.TestCase, _Persistent_Base):
         pass # Figure this out later
 
     def _makeCache(self, jar):
-        from persistent.cPickleCache import PickleCache
+        from persistent._compat import _c_optimizations_available as get_c
+        PickleCache = get_c()['persistent.picklecache'].PickleCache
         return PickleCache(jar)
 
 
@@ -2042,7 +2137,8 @@ class CPersistentTests(unittest.TestCase, _Persistent_Base):
 class Test_simple_new(unittest.TestCase):
 
     def _callFUT(self, x):
-        from persistent.cPersistence import simple_new
+        from persistent._compat import _c_optimizations_available as get_c
+        simple_new = get_c()['persistent.persistence'].simple_new
         return simple_new(x)
 
     def test_w_non_type(self):
